@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2014-2016 OpenMarket Ltd
 # Copyright 2018 New Vector Ltd
 #
@@ -496,7 +495,7 @@ def timeout_deferred(
 
         try:
             deferred.cancel()
-        except:  # noqa: E722, if we throw any exception it'll break time outs
+        except Exception:  # if we throw any exception it'll break time outs
             logger.exception("Canceller failed during timeout")
 
         # the cancel() call should have set off a chain of errbacks which
